import json
import webbrowser
import urllib

try:
    from urllib import quote  # Python 2.X
except ImportError:
    from urllib.parse import quote  # Python 3+

class Movie:

    def __init__(self,response):
        self.jsonObject = json.loads(response.text)
        self.trackers = [
            'udp://open.demonii.com:1337/announcejj',
            'udp://tracker.istole.it:80',
            'http://tracker.yify-torrents.com/announce',
            'udp://tracker.publicbt.com:80',
            'udp://tracker.openbittorrent.com:80',
            'udp://tracker.coppersurfer.tk:6969',
            'udp://glotorrents.pw:6969/announce',
            'udp://tracker.opentrackr.org:1337/announce',
            'udp://torrent.gresille.org:80/announce',
            'udp://p4p.arenabg.com:1337',
            'udp://tracker.leechers-paradise.org:6969',
            'http://exodus.desync.com:6969/announce'
        ]

        if "movie" in self.jsonObject["data"]:
            self.movie = self.jsonObject["data"]["movie"]
            self.qualities=[]
            for i in range(len(self.movie["torrents"])):
                self.qualities.append(self.movie['torrents'][i]['quality'])

    def Magnet(self,quality):

        base = "magnet:?xt=urn:btih:"+self.movie["torrents"][quality-1]["hash"]+"&dn"+quote(self.movie["title"])
        for t in self.trackers:
            base+= '&tr='+t

        return base

    def Torrent(self,quality):
        #print (self.movie['torrents'][quality-1]['url'],"~/Downloads/"+self.movie['title']+".torrent")

        return self.movie["torrents"][quality-1]['url']

    def getQualities(self):
        return self.qualities

    def getRecents(self):
        movie_list = []

        if self.jsonObject["data"]["movie_count"] > 0:
            for movie in self.jsonObject["data"]["movies"]:
                movie_list.append((str(movie["id"]).decode('unicode-escape'),str(movie["title"]).decode('unicode-escape')))

        return movie_list


    def details(self):
        movie = self.movie
        details = dict();

        details["title"] = str(movie["title_long"])
        details["year"] = str(movie["year"])
        details["rating"] = str(movie["rating"])
        details["runtime"] = str(movie["runtime"])
        details["language"]=str(movie["language"])
        details["genres"]=[str(genre) for genre in movie["genres"]]
        details["mpa_rating"]=movie["mpa_rating"]
        details["quality"]=str(self.getQualities())
        details["description_intro"]=movie["description_intro"]

        return details

    def getTrailerUrl(self):
        return "http://youtube.com/watch?v=" + self.movie["yt_trailer_code"]




import requests
try:
    from yifi import Movie
except ImportError:
    import Movie

try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode

class browser:
    def __init__(self):
        self.query ="https://yts.ag/api/v2/list_movies.json?"
        self.details = "https://yts.ag/api/v2/movie_details.json?"

    def bySeeds(self,limit=20,page=1,min_rating=0):
        param = dict({"limit":limit})
        param["page"]=page
        param["minimum_rating"]=min_rating

        response=requests.get(self.query+urlencode(param))

        movie = Movie.Movie(response)
        return movie.getRecents()

    def byRating(self,limit=20,page=1,min_rating=0):
        param = dict({"limit":limit})
        param["page"]=page
        param["minimum_rating"]=min_rating
        param["sort_by"]="rating"
        response=requests.get(self.query+urlencode(param))

        movie = Movie.Movie(response)
        return movie.getRecents()

    def detail(self,Id):
        param = dict({"movie_id":str(Id)})

        url = self.details+urlencode(param)
        response=requests.get(url)
        movie = Movie.Movie(response)

        return movie.details()

    def find(self,query):
        param = dict({"query_term":str(query)})
        url = self.query+urlencode(param)
        response=requests.get(url)
        movie = Movie.Movie(response)

        return movie.getRecents()

    def getTrailerUrl(self,Id):
        param = dict({"movie_id":str(Id)})

        url = self.details+urlencode(param)
        response=requests.get(url)
        movie = Movie.Movie(response)
        return str(movie.getTrailerUrl())

    def getQualities(self,Id):
        param = dict({"movie_id":str(Id)})

        url = self.details+urlencode(param)

        response=requests.get(url)
        movie = Movie.Movie(response)

        return movie.getQualities()

    def getMagnetUrl(self,Id,quality):
        param = dict({"movie_id":str(Id)})

        url = self.details+urlencode(param)
        response=requests.get(url)
        movie = Movie.Movie(response)
        return movie.Magnet(quality)

    def getTorrentUrl(self,Id,quality):
        param = dict({"movie_id":str(Id)})

        url = self.details+urlencode(param)
        response=requests.get(url)
        movie = Movie.Movie(response)
        return movie.Torrent(quality)



from __future__ import print_function
import webbrowser
import textwrap
import sys

try:
    from yifi import yifi
except ImportError:
    import yifi

def displayMovies(movies):
    for movie in movies:
        print(movie[0]+" "+movie[1])

def displayQualities(y,Id):
    print ("please select the quality :")
    qualities = y.getQualities(Id)

    for i in range(len(qualities)):
        print (i+1," -> ",str(qualities[i]))

    quality = int(input())
    return quality

def displayDetails(movie):
    print("title   : "+str(movie["title"]))
    print("year    : "+str(movie["year"]))
    print("rating  : * "+str(movie["rating"]))
    print("runtime : "+str(movie["runtime"])+" min")
    print("lang    : "+str(movie["language"]))
    print("genres  :",[str(genre) for genre in movie["genres"]])
    print("mpa     : "+movie["mpa_rating"])
    print("quality : " + movie["quality"])
    print("descr   :"+"\n\t  ".join(textwrap.wrap(" "+movie["description_intro"])))
    print("")


def help():
    f = open("help.txt")
    print(f.read())
    f.close()


def main():
    arg = sys.argv

    if '--help' in arg:
        help()
        exit()

    y = yifi.browser()

    if "-id" in arg:
        Id = ""
        if len(arg) >= arg.index("-id")+1:
            Id = arg[arg.index("-id")+1]

        if "--trailer" in arg:
            url = y.getTrailerUrl(Id)
            webbrowser.open(url)
            exit()

        if "--magnet" in arg:
            quality = ''
            if "-q" in arg:
                if len(arg) >= arg.index("-q")+1:
                    quality = arg[arg.index("-q")+1]

                if quality not in y.getQualities():
                    quality=displayQualities(y,Id)
            else:
                quality = displayQualities(y,Id)

            url = y.getMagnetUrl(Id,quality)
            webbrowser.open(url)
            exit()

        if "--download" in arg:

            quality = ""

            if "-q" in arg:

                if len(sys.arg) >= arg.index("-q")+1:
                    quality = sys.argv[sys.argv.index("-q")+1]

                if quality not in y.getQualities():
                    quality=displayQualities(y,Id);
            else:
                quality = displayQualities(y,Id)

            url = y.getTorrentUrl(Id,quality)
            webbrowser.open(url)
            exit()

        displayDetails(y.detail(Id))

    else:
        if "-f" in arg:
            q=""
            if len(arg) >= arg.index("-f")+1:
                q = arg[arg.index("-f")+1]

            if "-" not in q:
                lis = y.find(q)
                if(len(lis)==0):
                    print("sorry, no results found :(")
                else:
                    displayMovies(y.find(q))

        else:
            limit = 20
            page = 1
            min_rating = 0

            if "-l" in arg:
                if len(arg) >= arg.index("-l")+1:
                    limit = arg[arg.index("-l")+1]
                try:
                    val = int(limit)
                except ValueError:
                    print ("value error with parsing the limit ")
                    print ('-'*40)

            if "-p" in arg:
                if len(arg) >= arg.index("-p")+1:
                    page = arg[arg.index("-p")+1]
                try:
                    val = int(page)
                except ValueError:
                    print ("value error with parsing the page")
                    print ('-'*40)

            if "-m" in arg:
                if len(arg) >= arg.index("-m")+1:
                    min_rating = arg[arg.index("-m")+1]

            if "-s" in arg:
                if len(arg) >= arg.index("-s")+1:
                    sort = arg[arg.index("-s")+1]
                if sort == "r" or sort == "s" or sort == "rating" or sort == "seeds":
                    if sort == "r" or sort == "rating":
                        displayMovies(y.byRating(limit,page,min_rating))
                    else:
                        displayMovies(y.bySeeds(limit,page,min_rating))
                exit()

            displayMovies(y.bySeeds(limit,page,min_rating))



import os
from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__),fname)).read()

setup(
    name = "yifi",
    version = "0.1.7",
    author = "vitaminC",
    author_email = "dmakhil@gmail.com",
    url="https://github.com/AkhilMaskery/yifi",
    description = ("browse yifi on your command line"),
    entry_points={'console_scripts':['yifi=yifi.command_line:main']},
    license = "GPLv3",
    keywords = "yifi torrent download",
    packages = ['yifi'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Utilities',
    ],
)

